@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import atexit
+import base64
 import codecs
 import glob
 import json
@@ -19,7 +20,7 @@ from metakernel import MetaKernel, ProcessMetaKernel, REPLWrapper, u, MetaKernel
 from metakernel.pexpect import which
 from IPython.display import Image, SVG
 
-from . import __version__
+from ._version import __version__
 
 
 STDIN_PROMPT = '__stdin_prompt>'
@@ -36,6 +37,14 @@ HELP_LINKS = [
 
 ] + MetaKernel.help_links
 
+class PDF(object):
+    """Wrapper for PDF object for display.
+    """
+    def __init__(self, filename):
+        with open(filename, 'rb') as f:
+            data = f.read()
+            self._repr_pdf_ = base64.b64encode(data)
+
 
 def get_kernel_json():
     """Get the kernel json for the kernel.
@@ -52,12 +61,12 @@ def get_kernel_json():
 class OctaveKernel(ProcessMetaKernel):
     app_name = 'octave_kernel'
     implementation = 'Octave Kernel'
-    implementation_version = __version__,
+    implementation_version = __version__
     language = 'octave'
     help_links = HELP_LINKS
     kernel_json = Dict(get_kernel_json()).tag(config=True)
     cli_options = Unicode('').tag(config=True)
-    inline_toolkit = Unicode('gnuplot').tag(config=True)
+    inline_toolkit = Unicode('').tag(config=True)
 
     _octave_engine = None
     _language_version = None
@@ -88,6 +97,7 @@ class OctaveKernel(ProcessMetaKernel):
         if self._octave_engine:
             return self._octave_engine
         self._octave_engine = OctaveEngine(plot_settings=self.plot_settings,
+                                           defer_startup=True,
                                            error_handler=self.Error,
                                            stdin_handler=self.raw_input,
                                            stream_handler=self.Print,
@@ -106,6 +116,8 @@ class OctaveKernel(ProcessMetaKernel):
             self._octave_engine = None
             self.do_shutdown(True)
             return
+        if not self.octave_engine._has_startup:
+            self.octave_engine._startup()
         val = ProcessMetaKernel.do_execute_direct(self, code, silent=silent)
         if not silent:
             try:
@@ -161,7 +173,7 @@ class OctaveEngine(object):
     def __init__(self, error_handler=None, stream_handler=None,
                  line_handler=None,
                  stdin_handler=None, plot_settings=None,
-                 inline_toolkit='gnuplot',
+                 inline_toolkit=None, defer_startup = False,
                  cli_options='', logger=None):
         if not logger:
             logger = logging.getLogger(__name__)
@@ -175,7 +187,10 @@ class OctaveEngine(object):
         self.stream_handler = stream_handler
         self.stdin_handler = stdin_handler or sys.stdin
         self.line_handler = line_handler
-        self._startup(plot_settings)
+        self._has_startup = False
+        self._plot_settings = plot_settings
+        if not defer_startup:
+            self._startup()
         atexit.register(self._cleanup)
 
     @property
@@ -184,6 +199,9 @@ class OctaveEngine(object):
 
     @plot_settings.setter
     def plot_settings(self, settings):
+        if not self._has_startup:
+            self._default_toolkit = self.eval('graphics_toolkit', silent=True).split()[-1]
+
         settings = settings or dict(backend='inline')
         self._plot_settings = settings
 
@@ -203,8 +221,11 @@ class OctaveEngine(object):
         settings.setdefault('plot_dir', None)
 
         cmds = []
+
+        default_inline_toolkit = self.inline_toolkit or self._default_toolkit
+
         if settings['backend'] == 'inline':
-            cmds.append("graphics_toolkit('%s')" % self.inline_toolkit)
+            cmds.append("graphics_toolkit('%s')" % default_inline_toolkit)
             cmds.append("set(0, 'defaultfigurevisible', 'off');")
         elif settings['backend'].startswith('inline:'):
             backend = settings['backend'].replace('inline:', '')
@@ -264,6 +285,8 @@ class OctaveEngine(object):
             self.eval('drawnow("expose");')
             if not plot_dir:
                 return
+        if not self._has_startup:
+            self._startup()
         fmt = settings['format']
         res = settings['resolution']
         wid = settings['width']
@@ -305,6 +328,8 @@ class OctaveEngine(object):
             try:
                 if fname.lower().endswith('.svg'):
                     im = self._handle_svg(filename)
+                elif fname.lower().endswith('.pdf'):
+                    im = PDF(filename)
                 else:
                     im = Image(filename)
                 images.append(im)
@@ -317,14 +342,15 @@ class OctaveEngine(object):
             shutil.rmtree(plot_dir, True)
         return images
 
-    def _startup(self, plot_settings):
+    def _startup(self):
+        self._has_startup = True
         cwd = os.getcwd().replace(os.path.sep, '/')
         self._default_toolkit = self.eval('graphics_toolkit', silent=True).split()[-1]
         cmd = 'more off; source ~/.octaverc; cd("%s");%s'
         self.eval(cmd % (cwd, self.repl.prompt_change_cmd), silent=True)
         here = os.path.realpath(os.path.dirname(__file__))
         self.eval('addpath("%s")' % here.replace(os.path.sep, '/'))
-        self.plot_settings = plot_settings
+        self.plot_settings = self._plot_settings
 
     def _handle_svg(self, filename):
         """
@@ -372,7 +398,7 @@ class OctaveEngine(object):
 
     def _create_repl(self):
         cmd = self.executable
-        if 'octave-cli' not in cmd:
+        if 'octave' not in cmd:
             version_cmd = [self.executable, '--version']
             version = subprocess.check_output(version_cmd).decode('utf-8')
             if 'version 4' in version:
@@ -452,12 +478,14 @@ class OctaveEngine(object):
             fullpath = which(executable)
             if 'snap' not in fullpath:
                 executable = fullpath
-                if 'octave-cli' not in executable:
-                    raise OSError('OCTAVE_EXECUTABLE does not point to an octave-cli file, please see README')
+                if 'octave' not in executable:
+                    raise OSError('OCTAVE_EXECUTABLE does not point to an octave file, please see README')
         else:
             executable = which('octave-cli')
             if not executable:
-                raise OSError('octave-cli not found, please see README')
+                executable = which('octave')
+                if not executable:
+                    raise OSError('octave not found, please see README')
         return executable.replace(os.path.sep, '/')
 
     def _cleanup(self):
